##############################################
## Alt_NatlResources.R
## Purpose: Alternative Explanation: NatlResources
## Date: December 4, 2016 
##############################################

rm(list = ls()); gc()

##############################################
## SOURCING DISTANCE FUNCTIONS:
##############################################

# Sourcing C++ Functions:
invisible(sapply(c("Rcpp", "RcppArmadillo"), require, character.only = TRUE))
sourceCpp(paste0(top_dir, "/Code/DistanceFunctions.cpp"))

##############################################
## LOADING PREPPED SCAD DATA:
##############################################

setwd(data_dir)
load("SCAD/Prepared_SCAD.Rdata")

##############################################
## ONSHORE OIL W/IN 50KM:
##############################################

setwd(data_dir)
petro_shp <- readOGR(dsn = "PETRODATA V1.2", layer = "Petrodata_Onshore_V1.2")
petro_dt <- petro_shp@data %>% data.table()

# Coordinates for onshore fields:
petro_xy <- petro_dt[, .(LONG, LAT)] %>% as.matrix()

pb <- txtProgressBar(min = 0, max = nrow(scad_dt), style = 3)
for(i in 1:nrow(scad_dt)) {
    pt <- scad_dt[i, .(longitude, latitude)] %>% as.numeric()
    d_vec <- Dist(pt, petro_xy)

    # Fields within 50km
    petro_rows <- which(d_vec <= 50)
    # if(length(petro_rows) == 0) {next} else {break}

    if(length(petro_rows) == 0) {
        scad_dt[i, c("petro_disc", "petro_prod", "petro_tot") := 0L]
    } else {
        temp_dt <- petro_dt[petro_rows, .(DISC, PROD)]

        # Counting total fields and fields dicovered/producing before event:
        temp_dt <- temp_dt[, list(
            num_disc = sum(DISC <= scad_dt[i, eyr], na.rm = TRUE),
            num_prod = sum(PROD <= scad_dt[i, eyr], na.rm = TRUE),
            num_tot = .N)]

        scad_dt[i, petro_disc := temp_dt[, num_disc]]
        scad_dt[i, petro_prod := temp_dt[, num_prod]]
        scad_dt[i, petro_tot := temp_dt[, num_tot]]
    }
    rm(list = c("pt", "d_vec", "petro_rows", "temp_dt"))
    setTxtProgressBar(pb, i)
}

# plot(petro_shp)
# points(scad_dt[petro_tot > 0, .(longitude, latitude)])

##############################################
## DIAMONDS MINES W/IN 50KM:
##############################################

setwd(data_dir)
dir("Diamonds")
dia_shp <- readOGR(dsn = "Diamonds", layer = "DIADATA")
plot(dia_shp)

dia_dt <- dia_shp@data %>% data.table()
dia_dt[, disc_yr := year(ymd(DISC))]
dia_dt[, prod_yr := year(mdy(PROD))]

# Coordinates for onshore fields:
dia_xy <- dia_dt[, .(LONG, LAT)] %>% as.matrix()

pb <- txtProgressBar(min = 0, max = nrow(scad_dt), style = 3)
for(i in 1:nrow(scad_dt)) {
    pt <- scad_dt[i, .(longitude, latitude)] %>% as.numeric()
    d_vec <- Dist(pt, dia_xy)

    # Fields within 50km
    dia_rows <- which(d_vec <= 50)
    # if(length(dia_rows) == 0) {next} else {break}

    if(length(dia_rows) == 0) {
        scad_dt[i, c("dia_disc", "dia_prod", "dia_tot") := 0L]
    } else {
        temp_dt <- dia_dt[dia_rows, .(disc_yr, prod_yr)]

        # Counting total fields and fields dicovered/producing before event:
        temp_dt <- temp_dt[, list(
            num_disc = sum(disc_yr <= scad_dt[i, eyr], na.rm = TRUE),
            num_prod = sum(prod_yr <= scad_dt[i, eyr], na.rm = TRUE),
            num_tot = .N)]

        scad_dt[i, dia_disc := temp_dt[, num_disc]]
        scad_dt[i, dia_prod := temp_dt[, num_prod]]
        scad_dt[i, dia_tot := temp_dt[, num_tot]]
    }
    rm(list = c("pt", "d_vec", "dia_rows", "temp_dt"))
    setTxtProgressBar(pb, i)
}

# scad_dt[, summary(dia_tot)]
# plot(dia_shp)
# points(scad_dt[dia_tot > 0, .(longitude, latitude)])

scad_dt[, any_dia := as.integer(dia_tot > 0)]
scad_dt[, any_petro := as.integer(petro_tot > 0)]

##############################################
## REGRESSION MODELS:
##############################################

m1 <- felm(any_repress ~ ln_popdens + any_dia + any_petro |
    ccode + eyr | 0 | ccode,
    data = scad_dt)

m2 <- felm(any_repress ~ ln_popdens + any_dia + any_petro |
    leader_code + ccode + eyr +
    issue_type + govt_target + tactic + event_type
    | 0 | ccode,
    data = scad_dt)

m3 <- felm(lethal_repress ~ ln_popdens + any_dia + any_petro |
    ccode + eyr | 0 | ccode,
    data = scad_dt[any_repress == 1])

m4 <- felm(lethal_repress ~ ln_popdens + any_dia + any_petro |
    leader_code + ccode + eyr +
    issue_type + govt_target + tactic + event_type
    | 0 | ccode,
    data = scad_dt[any_repress == 1])

m5 <- felm(lethal_repress ~ ln_popdens + dia_tot + petro_tot |
    leader_code + ccode + eyr +
    issue_type + govt_target + tactic + event_type |
    0 | ccode,
    data = scad_dt[any_repress == 1])

##############################################
## ADDITIONAL MODEL INFORMATION
##############################################

m1_ct <- scad_dt[!is.na(any_repress) & !is.na(ln_popdens) &
    !is.na(ccode) & !is.na(eyr) & !is.na(any_dia) & !is.na(any_petro),
    list(nyear = length(unique(eyr)),
    ncntry = length(unique(ccode)),
    nldr = NA_integer_)]

m2_ct <- scad_dt[!is.na(any_repress) & !is.na(ln_popdens) &
    !is.na(groupname) & !is.na(eyr) & !is.na(any_dia) & !is.na(any_petro) &
    !is.na(issue_type) & !is.na(govt_target) & !is.na(tactic) & !is.na(event_type),
    list(nyear = length(unique(eyr)),
    ncntry = length(unique(ccode)),
    nldr = length(unique(leader_code)))]

m3_ct <- scad_dt[!is.na(lethal_repress) & !is.na(ln_popdens) &
    !is.na(ccode) & !is.na(eyr) & !is.na(any_dia) & !is.na(any_petro) &
    any_repress == 1,
    list(nyear = length(unique(eyr)),
    ncntry = length(unique(ccode)),
    nldr = NA_integer_)]

m4_ct <- scad_dt[!is.na(lethal_repress) & !is.na(ln_popdens) &
    !is.na(groupname) & !is.na(eyr) & !is.na(any_dia) & !is.na(any_petro) &
    !is.na(issue_type) & !is.na(govt_target) & !is.na(tactic) & !is.na(event_type) & any_repress == 1,
    list(nyear = length(unique(eyr)),
    ncntry = length(unique(ccode)),
    nldr = length(unique(leader_code)))]

m5_ct <- scad_dt[!is.na(lethal_repress) & !is.na(ln_popdens) &
    !is.na(groupname) & !is.na(eyr) & !is.na(dia_tot) & !is.na(petro_tot) &
    !is.na(issue_type) & !is.na(govt_target) & !is.na(tactic) & !is.na(event_type) & any_repress == 1,
    list(nyear = length(unique(eyr)),
    ncntry = length(unique(ccode)),
    nldr = length(unique(leader_code)))]

ct_dt <- rbindlist(list(m1_ct, m2_ct, m3_ct, m4_ct, m5_ct))

##############################################
## TABLE 14:
##############################################

# DV Labels:
depvars <- c("$\\mathbb{1}$(Repression)", "$\\mathbb{1}$(Lethal Repression)")

# Covariate Labels
covars <- c("Log(Pop. Density)", "$\\mathbb{1}$(Diamonds) (50km)", "$\\mathbb{1}$(Oil) (50km)", "$\\Sigma$ Diamonds (50km)", "$\\Sigma$ Oil (50km)")

# Counts for bottom of table:
cntry_FE <- c("Country FEs", format(ct_dt[, ncntry],
    big.mark = ",", trim = TRUE))
year_FE <- c("Year FEs", format(ct_dt[, nyear], big.mark = ",", trim = TRUE))
leader_FE <- c("Leader FEs", format(ct_dt[, nldr],
    big.mark = ",", trim = TRUE))

event_cntrls <- c("\\midrule Event Controls", "", "\\checkmark", "", "\\checkmark", "\\checkmark")

add_lines <- list(year_FE, cntry_FE, leader_FE, event_cntrls)
add_lines <- lapply(add_lines, function(x) {x[x == "NA"] <- ""; x})

setwd(tabs_dir)
stargazer(list(m1, m2, m3, m4, m5),
    dep.var.labels = depvars,
    covariate.labels = covars,
    keep.stat = "n",
    p.auto = TRUE,
    star.cutoffs = c(.1, .05),
    star.char = c("\\dagger", "*"),
    add.lines = add_lines,
    notes = "Robust std. errors clustered on country; $^{\\dagger} p <$ 0.1, $^*p <$ 0.05",
    notes.append = FALSE,
    table.placement = "ht!",
    digits = 3,
    label = "tab:alt_natlresources",
    title = "Pr(Repression) or Pr(Lethal) Given Proximity to Natural Resources",
    out = "Alt_NatlResources.tex")

##############################################
## TABLE 15:
##############################################

setwd(tabs_dir)
vars <- covars[2:5]
stargazer(scad_dt[!is.na(ln_popdens),
    .(any_dia, any_petro, dia_tot, petro_tot)],
    covariate.labels = vars,
    label = "tab:sumstats_alt_natlresources",
    title = "Summary Statistics: Pr(Repression) or Pr(Lethal) Given Proximity to Natural Resources",
    digits = 3,
    table.placement = "ht!",
    out = "SumStats_Alt_NatlResources.tex")
