#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;
using namespace arma;

// ------------------------------------------


double to_radians_cpp(double degrees){
    return(degrees * 3.141593 / 180);
}


// ------------------------------------------


// Haversine Formula:
double haversine_cpp(double lat1, double long1,
                     double lat2, double long2,
                     std::string unit="km"){
    int radius = 6378;
    double delta_phi = to_radians_cpp(lat2 - lat1);
    double delta_lambda = to_radians_cpp(long2 - long1);
    double phi1 = to_radians_cpp(lat1);
    double phi2 = to_radians_cpp(lat2);
    double term1 = pow(sin(delta_phi / 2), 2);
    double term2 = cos(phi1) * cos(phi2) * pow(sin(delta_lambda/2), 2);
    double the_terms = term1 + term2;
    double delta_sigma = 2 * atan2(sqrt(the_terms), sqrt(1-the_terms));
    double distance = radius * delta_sigma;

    /* if it is anything *but* km it is miles */
    if(unit != "km"){
        return(distance*0.621371);
    }

    return(distance);
}


// ------------------------------------------


// Distance Formula used by SH:
double sh_cpp(double lat1, double long1,
             double lat2, double long2) {
    double distance = pow(pow(111 * (lat1 - lat2), 2) + pow(cos(lat1 * 3.1415927 / 180) * 111 * (long1 - long2), 2), .5);
    return(distance);
}


// ------------------------------------------


// Minimum distance between point p and Matrix M:
// [[Rcpp::export]]
double MinDist(arma::vec p, arma::mat M,
   std::string dist_fn="Haversine"){

    long long int nrow = M.n_rows;
    arma::vec d_row(nrow);

    for( long long int i = 0; i < nrow; i++ ){
        if(dist_fn != "Haversine") {
            d_row[i] = sh_cpp(M(i,0), M(i,1), p[0], p[1]);
        } else {
            d_row[i] = haversine_cpp(M(i,0), M(i,1), p[0], p[1]);
        }
    }
    // Getting minimum value:
    NumericVector::iterator it = std::min_element(d_row.begin(), d_row.end());
    return *it;
}


// ------------------------------------------


// Distances between point p and Matrix M:
// [[Rcpp::export]]
arma::vec Dist(arma::vec p, arma::mat M,
   std::string dist_fn="Haversine"){

    long long int nrow = M.n_rows;
    arma::vec d_row(nrow);

    for( long long int i = 0; i < nrow; i++ ){
        if(dist_fn != "Haversine") {
            d_row[i] = sh_cpp(M(i,0), M(i,1), p[0], p[1]);
        } else {
            d_row[i] = haversine_cpp(M(i,0), M(i,1), p[0], p[1]);
        }
    }
    return d_row;
}


// ------------------------------------------


// Vector of minimum distance between Matrix P and Matrix M:
// [[Rcpp::export]]
arma::vec MinDistVec(arma::mat P, arma::mat M,
   std::string dist_fn="Haversine"){

    long long int p_rows = P.n_rows;
    arma::vec d_vec(p_rows);

    for( long long int j = 0; j < p_rows; j++ ){
        arma::vec p_vec(1);
        p_vec = P.row(j).t();

        d_vec[j] = MinDist(p_vec, M);
    }
    return d_vec;
}
